class PermTest(object):
    
    
    def __init__(self, data):
        """"""
        self.data = data
        self.formatted_data = None
        
        
        
    def format_data(self, response1=None, response2=None):
        """
        Converts data from a 2-column format (where each column refers to a treatment type) to
        a 2-column format where response is one column, treatment is the other.

        This is done for both efficiency of storage and test runtime.

        :param data: pandas DataFrame. each column must be of same length
        :return: pandas DataFrame in response, treatment format
        """
        # create new treatment Series
        
        if (response1==None) and (response2==None):
            # No columns specified, so default to first 2
            
            nrow, ncol = self.data.iloc[:,[0,1]].shape
            treatment_column = pd.Series([self.data.iloc[:,[0,1]].columns.values[i]
                            for i in range(ncol)
                            for j in range(nrow)], dtype='category')
            response_column = pd.concat([self.data.iloc[:,0],
                                         self.data.iloc[:,1]]).tolist()
        else:
            
            nrow, ncol = self.data.loc[:,[response1,response2]].shape
            treatment_column = pd.Series([self.data.loc[:,[response1,response2]].columns.values[i]
                            for i in range(ncol)
                            for j in range(nrow)], dtype='category')
            response_column = pd.concat([self.data.loc[:,response1],
                                         self.data.loc[:,response2]]).tolist()

        self.formatted_data = pd.DataFrame({"response": response_column,
                                      "treatment": treatment_column})

        
                
            
    def diff_means(self, response="response", treatment="treatment", data=None):
        """Calculates the difference of means between the two treatment groups"""
        if data is None:
            trt1 = self.formatted_data[treatment].cat.categories[0]
            trt2 = self.formatted_data[treatment].cat.categories[1]

            mean_diff = self.formatted_data[response][self.formatted_data[treatment]==trt1].mean() \
            - self.formatted_data[response][self.formatted_data[treatment]==trt2].mean()

            return mean_diff
        else:
            trt1 = data[treatment].cat.categories[0]
            trt2 = data[treatment].cat.categories[1]

            mean_diff = data[response][data[treatment]==trt1].mean() \
            - data[response][data[treatment]==trt2].mean()

            return mean_diff
    
    
    def simPermDsn(self, response="response", treatment="treatment", test="mean", k=100):
        """
        Creates permutation distributionn consisting of k values for our data
        :param response: column of response values
        :param treatment: column of treatment levels
        :param test: string detailing test to use. 
        :param k: integer specifying number of values in permuted distribution
        :return:
        """
        df = self.formatted_data[:]
        testStatistics = []
        if test == "mean":
            for simulation in range(k):
                df[treatment] = pd.Series(np.random.permutation(self.formatted_data[treatment]),
                                            dtype='category')
                testStatistics.append(self.diff_means(response=response, treatment=treatment, data=df))

            return testStatistics
        
    def pvalue(self, response="response", treatment="treatment", test="mean", k=100):
        """Returns p-value for our test"""
        if test == "mean":
            current_stat = self.diff_means(response=response, treatment=treatment)

            simulated_stats = self.simPermDsn(response=response, treatment=treatment, test=test, k=k)
            simulated_stats = np.array(simulated_stats)
            p_value = 1.0 * sum(abs(simulated_stats) >= abs(current_stat))/k

            return p_value
        
    def plot_dsn(self, response="response", treatment="treatment", test="mean", k=100):
        """Plots distribution of test statistics"""
        simulated_stats = self.simPermDsn(response=response, treatment=treatment, test=test, k=k)
        tit = "Permutated Test Statistic Distribution for k={} Simulations".format(k)
        plot_hist(simulated_stats, title=tit, xlabel="Test Statistic Value", ylabel="Frequency")




def chi_squared_statistic(o,e):
    """"""
    
    o = np.array(o)
    e = np.array(e)
    return np.sum(1.0*(o - e)**2 / e)


class ChiSquaredTest():
    
    def __init__(self, observed=None, expected=None):
        self.observed = observed
        self.expected = expected
        
        
    def plot_dsn(self, k=100):
        simulated_stats = self.simPermDsn(k=k)
        tit = "Chi-Squared Test Statistic Distribution for k={} Simulations".format(k)
        plot_hist(simulated_stats, title=tit, xlabel="Test Statistic Value", ylabel="Frequency")
    
    
    def testStat(self, input_observed=None, input_expected=None):
        if input_observed or input_expected:
            chisq_teststat = chi_squared_statistic(input_observed, input_expected)
        else:
            chisq_teststat = chi_squared_statistic(self.observed, self.expected)
        return chisq_teststat
    
    def simPermDsn(self, input_observed=None, input_expected=None, k=100):
        if input_observed or input_expected:
            test_stat = self.testStat(input_observed, input_expected)
            n = len(input_observed)
        else:
            test_stat = self.testStat()
            n = len(self.observed)
            
        # create sampling distribution
        chisqrd_vals = []
        for i in range(k):
            values = np.random.random((n,))
            ex = 1.0*n/2
            values[values<.5]=0
            values[values>=.5]=1
            diff1 = chi_squared_statistic(sum(values==0),ex)
            diff2 = chi_squared_statistic(sum(values==1),ex)
            chisqrd_vals.append(diff1+diff2)
        return chisqrd_vals
    
    def pvalue(self, input_observed=None, input_expected=None):
        
        current_stat = self.testStat(input_observed, input_expected)
        simulated_stats = self.simPermDsn(input_observed, input_expected)
        
        # p-value = proportion of test stats greater than ours
        p_value = 1.0*sum(simulated_stats >= current_stat) / len(simulated_stats)
        return p_value



from setuptools import setup

setup(name='twosampyle',
      version='0.1.1',
      description='Python module for two-sample statistical tests',
      url='https://github.com/jwilber/twosampyle',
      author='Jared Wilber',
      author_email='jdwlbr@gmail.com',
      license='GNU3',
      packages=['twosampyle'],
      install_requires=[
            'pandas',
            'numpy',
            'matplotlib'
      ],
      zip_safe=False)




import matplotlib.pyplot as plt
import matplotlib.cm as cm, matplotlib.font_manager as fm
# define the font styles
title_font = fm.FontProperties(family='serif', style='normal', size=19, weight='normal', stretch='normal')
label_font = fm.FontProperties(family='serif', style='normal', size=16, weight='normal', stretch='normal')
ticks_font = fm.FontProperties(family='serif', style='normal', size=10, weight='normal', stretch='normal')
annotation_font = fm.FontProperties(family='serif', style='normal', size=10, weight='normal', stretch='normal')
axis_bgcolor = 'white'
# make a re-usable function to draw nice bar charts
def plot_hist(data, title='', xlabel='Test Statistic Values', ylabel='Frequency', color='red'):
    
    if type(data) == list:
        data = pd.Series(data)
    ax = data.hist(figsize=[9, 6], width=0.35, alpha=0.5, 
                   color=color, edgecolor='k', grid=False, rwidth=.9)

    ax.set_xticklabels(data.index, rotation=45, rotation_mode='anchor', ha='right')
    ax.yaxis.grid(True)
    #for label in ax.get_yticklabels():
    #    label.set_fontproperties(ticks_font)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.grid(False)
    ax.set_facecolor(axis_bgcolor)   
    ax.set_title(title, fontproperties=title_font)
    ax.set_xlabel(xlabel, fontproperties=label_font)
    ax.set_ylabel(ylabel, fontproperties=label_font)

    plt.show()